import datetime
from .Exceptions import ParamError
from .types import MetricType, IndexType


def is_legal_host(host):
    if not isinstance(host, str):
        return False

    return True


def is_legal_port(port):
    if isinstance(port, str):
        try:
            _port = int(port)
        except Exception:
            return False

    if isinstance(port, int):
        _port = port

    if _port <= 0 or _port > 65535:
        return False

    return True


def is_legal_array(array):
    if not array or not isinstance(array, list):
        return False
    elif len(array) <= 0 or not isinstance(array[0], float):
        return False
    return True


def int_or_str(item):
    if isinstance(item, int):
        return str(item)
    else:
        return item


def is_correct_date_str(param):
    try:
        datetime.datetime.strptime(param, '%Y-%m-%d')
    except ValueError:
        raise ParamError('Incorrect data format, should be YYYY-MM-DD')
    return True


def is_legal_dimension(dim):
    return False if not isinstance(dim, int) or dim <= 0 else True


def is_legal_index_size(index_size):
    return False if not isinstance(index_size, int) or index_size <= 0 else True


def is_legal_metric_type(metric_type):
    if isinstance(metric_type, int):
        try:
            _metric_type = MetricType(metric_type)
        except Exception:
            return False

    return True


def is_legal_index_type(index_type):
    if isinstance(index_type, int):
        try:
            index_type = IndexType(index_type)
        except Exception:
            return False

    if isinstance(index_type, IndexType):
        if index_type != IndexType.INVALID:
            return True

    return False


def is_legal_table_name(table_name):
    return False if not isinstance(table_name, str) or len(table_name) == 0 else True


def is_legal_nlist(nlist):
    return False if not isinstance(nlist, int) or nlist <= 0 else True


def is_legal_topk(topk):
    return False if not isinstance(topk, int) or topk <= 0 else True


def is_legal_ids(ids):
    return False if not isinstance(ids, list) or not isinstance(ids[0], int) else True


def is_legal_nprobe(nprobe):
    return False if not isinstance(nprobe, int) or nprobe <= 0 else True


def is_legal_cmd(cmd):
    return False if not isinstance(cmd, str) or len(cmd) == 0 else True


def parser_range_date(date):
    if isinstance(date, datetime.date):
        return date.strftime('%Y-%m-%d')
    elif isinstance(date, str):
        if not is_correct_date_str(date):
            raise ParamError('Date string should be YY-MM-DD format!')
        else:
            return date
    else:
        raise ParamError(
            'Date should be YY-MM-DD format string or datetime.date, '
            'or datetime.datetime object')


def is_legal_date_range(start, end):
    start_date = datetime.datetime.strptime(start, "%Y-%m-%d")
    end_date = datetime.datetime.strptime(end, "%Y-%m-%d")
    if (end_date - start_date).days < 0:
        return False
    else:
        return True


def _raise_param_error(param_name):
    raise ParamError("{} is illegal".format(param_name))


def check_pass_param(*args, **kwargs):
    if kwargs is None:
        raise ParamError("Param should not be None")

    for key, value in kwargs.items():
        if key in ("table_name",):
            if not is_legal_table_name(value):
                _raise_param_error(key)
        elif key == "dimension":
            if not is_legal_dimension(value):
                _raise_param_error(key)
        elif key in ("index_type",):
            if not is_legal_index_type(value):
                _raise_param_error(key)
        elif key == "index_file_size":
            if not is_legal_index_size(value):
                _raise_param_error(key)
        elif key == "metric_type":
            if not is_legal_metric_type(value):
                _raise_param_error(key)
        elif key in ("topk", "top_k"):
            if not is_legal_topk(value):
                _raise_param_error(key)
        elif key in ("ids",):
            if not is_legal_ids(value):
                _raise_param_error(key)
        elif key in ("nprobe",):
            if not is_legal_nprobe(value):
                _raise_param_error(key)
        elif key in ("nlist",):
            if not is_legal_nlist(value):
                _raise_param_error(key)
        elif key in ("cmd",):
            if not is_legal_cmd(value):
                _raise_param_error(key)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import milvus_pb2 as milvus__pb2
from . import status_pb2 as status__pb2


class MilvusServiceStub(object):
    # missing associated documentation comment in .proto file
    pass

    def __init__(self, channel):
        """Constructor.

        Args:
          channel: A grpc.Channel.
        """
        self.CreateTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/CreateTable',
            request_serializer=milvus__pb2.TableSchema.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.HasTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/HasTable',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=milvus__pb2.BoolReply.FromString,
        )
        self.DropTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/DropTable',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.CreateIndex = channel.unary_unary(
            '/milvus.grpc.MilvusService/CreateIndex',
            request_serializer=milvus__pb2.IndexParam.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.Insert = channel.unary_unary(
            '/milvus.grpc.MilvusService/Insert',
            request_serializer=milvus__pb2.InsertParam.SerializeToString,
            response_deserializer=milvus__pb2.VectorIds.FromString,
        )
        self.Search = channel.unary_unary(
            '/milvus.grpc.MilvusService/Search',
            request_serializer=milvus__pb2.SearchParam.SerializeToString,
            response_deserializer=milvus__pb2.TopKQueryResultList.FromString,
        )
        self.SearchInFiles = channel.unary_unary(
            '/milvus.grpc.MilvusService/SearchInFiles',
            request_serializer=milvus__pb2.SearchInFilesParam.SerializeToString,
            response_deserializer=milvus__pb2.TopKQueryResultList.FromString,
        )
        self.DescribeTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/DescribeTable',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=milvus__pb2.TableSchema.FromString,
        )
        self.CountTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/CountTable',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=milvus__pb2.TableRowCount.FromString,
        )
        self.ShowTables = channel.unary_unary(
            '/milvus.grpc.MilvusService/ShowTables',
            request_serializer=milvus__pb2.Command.SerializeToString,
            response_deserializer=milvus__pb2.TableNameList.FromString,
        )
        self.Cmd = channel.unary_unary(
            '/milvus.grpc.MilvusService/Cmd',
            request_serializer=milvus__pb2.Command.SerializeToString,
            response_deserializer=milvus__pb2.StringReply.FromString,
        )
        self.DeleteByRange = channel.unary_unary(
            '/milvus.grpc.MilvusService/DeleteByRange',
            request_serializer=milvus__pb2.DeleteByRangeParam.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.PreloadTable = channel.unary_unary(
            '/milvus.grpc.MilvusService/PreloadTable',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )
        self.DescribeIndex = channel.unary_unary(
            '/milvus.grpc.MilvusService/DescribeIndex',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=milvus__pb2.IndexParam.FromString,
        )
        self.DropIndex = channel.unary_unary(
            '/milvus.grpc.MilvusService/DropIndex',
            request_serializer=milvus__pb2.TableName.SerializeToString,
            response_deserializer=status__pb2.Status.FromString,
        )


class MilvusServiceServicer(object):
    # missing associated documentation comment in .proto file
    pass

    def CreateTable(self, request, context):
        """*
        @brief Create table method

        This method is used to create table

        @param param, use to provide table information to be created.

        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HasTable(self, request, context):
        """*
        @brief Test table existence method

        This method is used to test table existence.

        @param table_name, table name is going to be tested.

        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DropTable(self, request, context):
        """*
        @brief Delete table method

        This method is used to delete table.

        @param table_name, table name is going to be deleted.

        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateIndex(self, request, context):
        """*
        @brief Build index by table method

        This method is used to build index by table in sync mode.

        @param table_name, table is going to be built index.

        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Insert(self, request, context):
        """*
        @brief Add vector array to table

        This method is used to add vector array to table.

        @param table_name, table_name is inserted.
        @param record_array, vector array is inserted.

        @return vector id array
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Search(self, request, context):
        """*
        @brief Query vector

        This method is used to query vector in table.

        @param table_name, table_name is queried.
        @param query_record_array, all vector are going to be queried.
        @param query_range_array, optional ranges for conditional search. If not specified, search whole table
        @param topk, how many similarity vectors will be searched.

        @return query result array.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SearchInFiles(self, request, context):
        """*
        @brief Internal use query interface

        This method is used to query vector in specified files.

        @param file_id_array, specified files id array, queried.
        @param query_record_array, all vector are going to be queried.
        @param query_range_array, optional ranges for conditional search. If not specified, search whole table
        @param topk, how many similarity vectors will be searched.

        @return query result array.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeTable(self, request, context):
        """*
        @brief Get table schema

        This method is used to get table schema.

        @param table_name, target table name.

        @return table schema
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CountTable(self, request, context):
        """*
        @brief Get table schema

        This method is used to get table schema.

        @param table_name, target table name.

        @return table schema
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ShowTables(self, request, context):
        """*
        @brief List all tables in database

        This method is used to list all tables.


        @return table names.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Cmd(self, request, context):
        """*
        @brief Give the server status

        This method is used to give the server status.

        @return Server status.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteByRange(self, request, context):
        """*
        @brief delete table by range

        This method is used to delete vector by range

        @return rpc status.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PreloadTable(self, request, context):
        """*
        @brief preload table

        This method is used to preload table

        @return Status.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeIndex(self, request, context):
        """*
        @brief describe index

        This method is used to describe index

        @return Status.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DropIndex(self, request, context):
        """*
        @brief drop index

        This method is used to drop index

        @return Status.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MilvusServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'CreateTable': grpc.unary_unary_rpc_method_handler(
            servicer.CreateTable,
            request_deserializer=milvus__pb2.TableSchema.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'HasTable': grpc.unary_unary_rpc_method_handler(
            servicer.HasTable,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=milvus__pb2.BoolReply.SerializeToString,
        ),
        'DropTable': grpc.unary_unary_rpc_method_handler(
            servicer.DropTable,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'CreateIndex': grpc.unary_unary_rpc_method_handler(
            servicer.CreateIndex,
            request_deserializer=milvus__pb2.IndexParam.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'Insert': grpc.unary_unary_rpc_method_handler(
            servicer.Insert,
            request_deserializer=milvus__pb2.InsertParam.FromString,
            response_serializer=milvus__pb2.VectorIds.SerializeToString,
        ),
        'Search': grpc.unary_unary_rpc_method_handler(
            servicer.Search,
            request_deserializer=milvus__pb2.SearchParam.FromString,
            response_serializer=milvus__pb2.TopKQueryResultList.SerializeToString,
        ),
        'SearchInFiles': grpc.unary_unary_rpc_method_handler(
            servicer.SearchInFiles,
            request_deserializer=milvus__pb2.SearchInFilesParam.FromString,
            response_serializer=milvus__pb2.TopKQueryResultList.SerializeToString,
        ),
        'DescribeTable': grpc.unary_unary_rpc_method_handler(
            servicer.DescribeTable,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=milvus__pb2.TableSchema.SerializeToString,
        ),
        'CountTable': grpc.unary_unary_rpc_method_handler(
            servicer.CountTable,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=milvus__pb2.TableRowCount.SerializeToString,
        ),
        'ShowTables': grpc.unary_unary_rpc_method_handler(
            servicer.ShowTables,
            request_deserializer=milvus__pb2.Command.FromString,
            response_serializer=milvus__pb2.TableNameList.SerializeToString,
        ),
        'Cmd': grpc.unary_unary_rpc_method_handler(
            servicer.Cmd,
            request_deserializer=milvus__pb2.Command.FromString,
            response_serializer=milvus__pb2.StringReply.SerializeToString,
        ),
        'DeleteByRange': grpc.unary_unary_rpc_method_handler(
            servicer.DeleteByRange,
            request_deserializer=milvus__pb2.DeleteByRangeParam.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'PreloadTable': grpc.unary_unary_rpc_method_handler(
            servicer.PreloadTable,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
        'DescribeIndex': grpc.unary_unary_rpc_method_handler(
            servicer.DescribeIndex,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=milvus__pb2.IndexParam.SerializeToString,
        ),
        'DropIndex': grpc.unary_unary_rpc_method_handler(
            servicer.DropIndex,
            request_deserializer=milvus__pb2.TableName.FromString,
            response_serializer=status__pb2.Status.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'milvus.grpc.MilvusService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))



from .utils import *

from milvus.client.Exceptions import (
    ParamError
)

from .types import MetricType, IndexType


class TableSchema(object):
    """
    Table Schema

    :type  table_name: str
    :param table_name: (Required) name of table

        `IndexType`: 0-invalid, 1-flat, 2-ivflat, 3-IVF_SQ8, 4-MIX_NSG

    :type  dimension: int64
    :param dimension: (Required) dimension of vector

    :type  index_file_size: int64
    :param index_file_size: (Optional) max size of files which store index

    :type  metric_type: MetricType
    :param metric_type: (Optional) vectors metric type

        `MetricType`: 1-L2, 2-IP

    """

    def __init__(self, table_name, dimension, index_file_size, metric_type):

        # TODO may raise UnicodeEncodeError
        if table_name is None:
            raise ParamError('Table name can\'t be None')
        table_name = str(table_name) if not isinstance(table_name, str) else table_name
        if not is_legal_dimension(dimension):
            raise ParamError('Illegal dimension, effective range: (0 , 16384]')
        if isinstance(metric_type, int):
            metric_type = MetricType(metric_type)
        if not isinstance(metric_type, MetricType):
            raise ParamError('Illegal metric_type, should be MetricType')

        self.table_name = table_name
        self.dimension = dimension
        self.index_file_size = index_file_size
        self.metric_type = metric_type

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))


class Range(object):
    """
    Range information

    :type  start_date: str, date or datetime

        `str should be YY-MM-DD format, e.g. "2019-07-01"`

    :param start_date: Range start date

    :type  end_date: str, date or datetime

        `str should be YY-MM-DD format, e.g. "2019-07-01"`

    :param end_date: Range end date

    """

    def __init__(self, start_date, end_date):
        start_date = parser_range_date(start_date)
        end_date = parser_range_date(end_date)
        if is_legal_date_range(start_date, end_date):
            self.start_date = start_date
            self.end_date = end_date
        else:
            raise ParamError("The start-date should be smaller"
                             " than or equal to end-date!")


class TopKQueryResult(object):
    """
    TopK query results, 2-D array of query result
    """

    def __init__(self, raw_source):
        self._raw = raw_source
        self._array = self._create_array(self._raw)

    def _create_array(self, raw):

        array = []
        for topk_result in raw.topk_query_result:
            topk_result_list = []
            topk_result_list.extend(topk_result.query_result_arrays)
            array.append(topk_result_list)

        return array

    @property
    def shape(self):
        row = len(self._array)

        if row == 0:
            column = 0
        else:
            column = len(self._array[0])

        return row, column

    def __getitem__(self, item):
        return self._array.__getitem__(item)

    def __iter__(self):
        return self._array.__iter__()

    def __len__(self):
        return self._array.__len__()

    def __repr__(self):
        """

        :return:
        """

        lam = lambda x: "(id:{}, distance:{})".format(x.id, x.distance)

        if self.__len__() > 10:
            middle = ''

            for topk in self[:3]:
                middle = middle + " [ %s" % ",\n   ".join(map(lam, topk[:3]))
                middle += ",\n   ..."
                middle += "\n   %s ]\n\n" % lam(topk[-1])

            spaces = """        ......
        ......"""

            ahead = "[\n%s%s\n]" % (middle, spaces)
            return ahead
        else:
            str_out_list = []
            for i in range(self.__len__()):
                str_out_list.append("[\n%s\n]" % ",\n".join(map(lam, self[i])))

            return "[\n%s\n]" % ",\n".join(str_out_list)


class IndexParam(object):
    """
    Index Param

    :type  table_name: str
    :param table_name: (Required) name of table

    :type  index_type: IndexType
    :param index_type: (Required) index type, default = IndexType.INVALID

        `IndexType`: 0-invalid, 1-flat, 2-ivflat, 3-IVF_SQ8, 4-MIX_NSG

    :type  nlist: int64
    :param nlist: (Required) num of cell

    """

    def __init__(self, table_name, index_type, nlist):

        if table_name is None:
            raise ParamError('Table name can\'t be None')
        table_name = str(table_name) if not isinstance(table_name, str) else table_name

        if isinstance(index_type, int):
            index_type = IndexType(index_type)
        if not isinstance(index_type, IndexType) or index_type == IndexType.INVALID:
            raise ParamError('Illegal index_type, should be IndexType but not IndexType.INVALID')

        self._table_name = table_name
        self._index_type = index_type
        self._nlist = nlist

    def __str__(self):
        L = ['%s=%r' % (key.lstrip('_'), value)
             for key, value in self.__dict__.items()]
        return '(%s)' % (', '.join(L))

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))


def _abstract():
    raise NotImplementedError('You need to override this function')


class ConnectIntf(object):
    """SDK client abstract class

    Connection is a abstract class

    """

    def connect(self, host=None, port=None, uri=None):
        """
        Connect method should be called before any operations
        Server will be connected after connect return OK
        Should be implemented

        :type  host: str
        :param host: host

        :type  port: str
        :param port: port

        :type  uri: str
        :param uri: (Optional) uri

        :return: Status,  indicate if connect is successful
        """
        _abstract()

    def connected(self):
        """
        connected, connection status
        Should be implemented

        :return: Status,  indicate if connect is successful
        """
        _abstract()

    def disconnect(self):
        """
        Disconnect, server will be disconnected after disconnect return SUCCESS
        Should be implemented

        :return: Status,  indicate if connect is successful
        """
        _abstract()

    def create_table(self, param):
        """
        Create table
        Should be implemented

        :type  param: TableSchema
        :param param: provide table information to be created

        :return: Status, indicate if connect is successful
        """
        _abstract()

    def has_table(self, table_name):
        """

        This method is used to test table existence.
        Should be implemented

        :param table_name: table name is going to be tested.
        :type table_name: str

        :return:
            has_table: bool, if given table_name exists

        """
        _abstract()

    def delete_table(self, table_name):
        """
        Delete table
        Should be implemented

        :type  table_name: str
        :param table_name: table_name of the deleting table

        :return: Status, indicate if connect is successful
        """
        _abstract()

    def add_vectors(self, table_name, records):
        """
        Add vectors to table
        Should be implemented

        :type  table_name: str
        :param table_name: table name been inserted

        :type  records: list[RowRecord]
        :param records: list of vectors been inserted

        :returns:
            Status : indicate if vectors inserted successfully
            ids :list of id, after inserted every vector is given a id
        """
        _abstract()

    def search_vectors(self, table_name, query_records, query_ranges, top_k):
        """
        Query vectors in a table
        Should be implemented

        :type  table_name: str
        :param table_name: table name been queried

        :type  query_records: list[RowRecord]
        :param query_records: all vectors going to be queried

        :type  query_ranges: list[Range]
        :param query_ranges: Optional ranges for conditional search.
            If not specified, search whole table

        :type  top_k: int
        :param top_k: how many similar vectors will be searched

        :returns:
            Status:  indicate if query is successful
            query_results: list[TopKQueryResult]
        """
        _abstract()

    def search_vectors_in_files(self, table_name, file_ids, query_records, query_ranges, top_k):
        """
        Query vectors in a table, query vector in specified files
        Should be implemented

        :type  table_name: str
        :param table_name: table name been queried

        :type  file_ids: list[str]
        :param file_ids: Specified files id array

        :type  query_records: list[RowRecord]
        :param query_records: all vectors going to be queried

        :type  query_ranges: list[Range]
        :param query_ranges: Optional ranges for conditional search.
            If not specified, search whole table

        :type  top_k: int
        :param top_k: how many similar vectors will be searched

        :returns:
            Status:  indicate if query is successful
            query_results: list[TopKQueryResult]
        """
        _abstract()

    def describe_table(self, table_name):
        """
        Show table information
        Should be implemented

        :type  table_name: str
        :param table_name: which table to be shown

        :returns:
            Status: indicate if query is successful
            table_schema: TableSchema, given when operation is successful
        """
        _abstract()

    def get_table_row_count(self, table_name):
        """
        Get table row count
        Should be implemented

        :type  table_name, str
        :param table_name, target table name.

        :returns:
            Status: indicate if operation is successful
            count: int, table row count
        """
        _abstract()

    def show_tables(self):
        """
        Show all tables in database
        should be implemented

        :return:
            Status: indicate if this operation is successful
            tables: list[str], list of table names
        """
        _abstract()

    def create_index(self, table_name, index):
        """
        Create specified index in a table
        should be implemented

        :type  table_name: str
        :param table_name: table name

         :type index: dict
        :param index: index information dict

            example: index = {
                "index_type": IndexType.FLAT,
                "nlist": 18384
            }

        :return:
            Status: indicate if this operation is successful

        :rtype: Status
        """
        _abstract()

    def client_version(self):
        """
        Provide client version
        should be implemented

        :return:
            Status: indicate if operation is successful

            str : Client version

        :rtype: (Status, str)
        """
        _abstract()

    def server_version(self):
        """
        Provide server version
        should be implemented

        :return:
            Status: indicate if operation is successful

            str : Server version

        :rtype: (Status, str)
        """
        _abstract()

    def server_status(self):
        """
        Provide server status. When cmd !='version', provide 'OK'
        should be implemented

        :return:
            Status: indicate if operation is successful

            str : Server version

        :rtype: (Status, str)
        """
        _abstract()

    def delete_vectors_by_range(self, table_name, start_time, end_time):
        """
        delete vector by date range. The data range contains start_time but not end_time
        should be implemented

        :param table_name: table name
        :type  table_name: str

        :param start_time: range start time
        :type  start_time: str, date, datetime

        :param end_time: range end time(not contains in range)
        :type  end_time: str, date, datetime

        :return:
        """

        _abstract()

    def preload_table(self, table_name):
        """
        load table to cache in advance
        should be implemented

        :param table_name: target table name.
        :type table_name: str

        :return:
            Status: indicate if operation is successful

        ：:rtype: Status
        """

        _abstract()

    def describe_index(self, table_name):
        """
        Show index information
        should be implemented

        :param table_name: target table name.
        :type table_name: str

        :return:
            Status: indicate if operation is successful

            TableSchema: table detail information

        :rtype: (Status, TableSchema)
        """

        _abstract()

    def drop_index(self, table_name):
        """
        Show index information
        should be implemented

        :param table_name: target table name.
        :type table_name: str

        :return:
            Status: indicate if operation is successful

        ：:rtype: Status
        """

        _abstract()



from .client.GrpcClient import GrpcMilvus as Milvus
from .client.GrpcClient import Prepare
from .client.types import IndexType, MetricType, Status
from .client import Exceptions as milvusError
from .client import __version__

__all__ = ['Milvus', 'Prepare', 'Status', 'IndexType', 'MetricType', 'milvusError', '__version__']
